@@ -34,18 +34,28 @@
 #endif
 
 static FILE *logfile;
+static char *logbuffer;
+static size_t logbufsize;
+static size_t logsize;
+
+#define LOGBUFSIZ 1024
 
 static bool
 init_log(void)
 {
 	extern struct conf *conf;
 
-	if (logfile) {
+	if (logbuffer || logfile) {
 		return true;
 	}
 	assert(conf);
+	if (conf->debug) {
+		logbufsize = LOGBUFSIZ;
+		logbuffer = x_malloc(logbufsize);
+		logsize = 0;
+	}
 	if (str_eq(conf->log_file, "")) {
-		return false;
+		return conf->debug;
 	}
 	logfile = fopen(conf->log_file, "a");
 	if (logfile) {
@@ -58,12 +68,23 @@ init_log(void)
 	}
 }
 
+static void
+append_log(const char *s, size_t len)
+{
+	assert(logbuffer);
+	if (logsize + len + 1 > logbufsize) {
+		logbufsize = logbufsize + len + 1 + LOGBUFSIZ;
+		logbuffer = x_realloc(logbuffer, logbufsize);
+	}
+	memcpy(logbuffer + logsize, s, len);
+	logsize += len;
+}
+
 static void
 log_prefix(bool log_updated_time)
 {
-#ifdef HAVE_GETTIMEOFDAY
 	static char prefix[200];
-
+#ifdef HAVE_GETTIMEOFDAY
 	if (log_updated_time) {
 		char timestamp[100];
 		struct tm *tm;
@@ -78,10 +99,15 @@ log_prefix(bool log_updated_time)
 		snprintf(prefix, sizeof(prefix),
 		         "[%s.%06d %-5d] ", timestamp, (int)tv.tv_usec, (int)getpid());
 	}
-	fputs(prefix, logfile);
 #else
-	fprintf(logfile, "[%-5d] ", (int)getpid());
+	snprintf(prefix, sizeof(prefix), "[%-5d] ", (int)getpid());
 #endif
+	if (logfile) {
+		fputs(prefix, logfile);
+	}
+	if (logbuffer) {
+		append_log(prefix, strlen(prefix));
+	}
 }
 
 static long
@@ -99,6 +125,8 @@ path_max(const char *path)
 #endif
 }
 
+static void warn_log_fail(void) ATTR_NORETURN;
+
 // Warn about failure writing to the log file and then exit.
 static void
 warn_log_fail(void)
@@ -118,12 +146,23 @@ vlog(const char *format, va_list ap, bool log_updated_time)
 		return;
 	}
 
+	va_list aq;
+	va_copy(aq, ap);
 	log_prefix(log_updated_time);
-	int rc1 = vfprintf(logfile, format, ap);
-	int rc2 = fprintf(logfile, "\n");
-	if (rc1 < 0 || rc2 < 0) {
-		warn_log_fail();
+	if (logfile) {
+		int rc1 = vfprintf(logfile, format, ap);
+		int rc2 = fprintf(logfile, "\n");
+		if (rc1 < 0 || rc2 < 0) {
+			warn_log_fail();
+		}
 	}
+	if (logbuffer) {
+		char buf[1024];
+		size_t len = vsnprintf(buf, sizeof(buf), format, aq);
+		append_log(buf, len);
+		append_log("\n", 1);
+	}
+	va_end(aq);
 }
 
 // Write a message to the log file (adding a newline) and flush.
@@ -159,12 +198,29 @@ cc_log_argv(const char *prefix, char **argv)
 	}
 
 	log_prefix(true);
-	fputs(prefix, logfile);
-	print_command(logfile, argv);
-	int rc = fflush(logfile);
-	if (rc) {
-		warn_log_fail();
+	if (logfile) {
+		fputs(prefix, logfile);
+		print_command(logfile, argv);
+		int rc = fflush(logfile);
+		if (rc) {
+			warn_log_fail();
+		}
 	}
+	if (logbuffer) {
+		append_log(prefix, strlen(prefix));
+		char *s = format_command(argv);
+		append_log(s, strlen(s));
+		free(s);
+	}
+}
+
+// Copy the current log memory buffer to an output file.
+void
+cc_dump_log_buffer(const char *path)
+{
+	FILE *file = fopen(path, "w");
+	(void) fwrite(logbuffer, 1, logsize, file);
+	fclose(file);
 }
 
 // Something went badly wrong!
@@ -216,7 +272,14 @@ copy_fd(int fd_in, int fd_out)
 int
 mkstemp(char *template)
 {
+#ifdef __GNUC__
+	#pragma GCC diagnostic push
+	#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
 	mktemp(template);
+#ifdef __GNUC__
+	#pragma GCC diagnostic pop
+#endif
 	return open(template, O_RDWR | O_CREAT | O_EXCL | O_BINARY, 0600);
 }
 #endif
@@ -523,16 +586,16 @@ get_hostname(void)
 		DWORD dw = WSAGetLastError();
 
 		FormatMessage(
-		  FORMAT_MESSAGE_ALLOCATE_BUFFER |
-		  FORMAT_MESSAGE_FROM_SYSTEM |
-		  FORMAT_MESSAGE_IGNORE_INSERTS,
-		  NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-		  (LPTSTR) &lp_msg_buf, 0, NULL);
+			FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM |
+			FORMAT_MESSAGE_IGNORE_INSERTS,
+			NULL, dw, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+			(LPTSTR) &lp_msg_buf, 0, NULL);
 
 		LPVOID lp_display_buf = (LPVOID) LocalAlloc(
-		  LMEM_ZEROINIT,
-		  (lstrlen((LPCTSTR) lp_msg_buf) + lstrlen((LPCTSTR) __FILE__) + 200)
-		  * sizeof(TCHAR));
+			LMEM_ZEROINIT,
+			(lstrlen((LPCTSTR) lp_msg_buf) + lstrlen((LPCTSTR) __FILE__) + 200)
+			* sizeof(TCHAR));
 		_snprintf((LPTSTR) lp_display_buf,
 		          LocalSize(lp_display_buf) / sizeof(TCHAR),
 		          TEXT("%s failed with error %lu: %s"), __FILE__, dw,
@@ -578,11 +641,11 @@ format_hash_as_string(const unsigned char *hash, int size)
 	return ret;
 }
 
-char const CACHEDIR_TAG[] =
-  "Signature: 8a477f597d28d172789f06886806bc55\n"
-  "# This file is a cache directory tag created by ccache.\n"
-  "# For information about cache directory tags, see:\n"
-  "#\thttp://www.brynosaurus.com/cachedir/\n";
+static char const CACHEDIR_TAG[] =
+	"Signature: 8a477f597d28d172789f06886806bc55\n"
+	"# This file is a cache directory tag created by ccache.\n"
+	"# For information about cache directory tags, see:\n"
+	"#\thttp://www.brynosaurus.com/cachedir/\n";
 
 int
 create_cachedirtag(const char *dir)
@@ -717,6 +780,16 @@ x_realloc(void *ptr, size_t size)
 	return p2;
 }
 
+// This is like setenv.
+void x_setenv(const char *name, const char *value)
+{
+#ifdef HAVE_SETENV
+	setenv(name, value, true);
+#else
+	putenv(format("%s=%s", name, value)); // Leak to environment.
+#endif
+}
+
 // This is like unsetenv.
 void x_unsetenv(const char *name)
 {
@@ -775,9 +848,6 @@ reformat(char **ptr, const char *format, ...)
 	}
 	va_end(ap);
 
-	if (!ptr) {
-		fatal("Out of memory in reformat");
-	}
 	if (saved) {
 		free(saved);
 	}
@@ -983,7 +1053,7 @@ parse_size_with_suffix(const char *str, uint64_t *size)
 		// Default suffix: G.
 		x *= 1000 * 1000 * 1000;
 	}
-	*size = x;
+	*size = (uint64_t)x;
 	return true;
 }
 
@@ -1006,7 +1076,7 @@ static BOOL GetFileNameFromHandle(HANDLE file_handle, TCHAR *filename,
 
 	// Create a file mapping object.
 	HANDLE file_map =
-	  CreateFileMapping(file_handle, NULL, PAGE_READONLY, 0, 1, NULL);
+		CreateFileMapping(file_handle, NULL, PAGE_READONLY, 0, 1, NULL);
 	if (!file_map) {
 		return FALSE;
 	}
@@ -1084,8 +1154,8 @@ x_realpath(const char *path)
 		path++;  // Skip leading slash.
 	}
 	HANDLE path_handle = CreateFile(
-	  path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
-	  FILE_ATTRIBUTE_NORMAL, NULL);
+		path, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING,
+		FILE_ATTRIBUTE_NORMAL, NULL);
 	if (INVALID_HANDLE_VALUE != path_handle) {
 #ifdef HAVE_GETFINALPATHNAMEBYHANDLEW
 		GetFinalPathNameByHandle(path_handle, ret, maxlen, FILE_NAME_NORMALIZED);
@@ -1431,18 +1501,18 @@ x_rename(const char *oldpath, const char *newpath)
 		LPVOID lp_msg_buf;
 		DWORD dw = GetLastError();
 		FormatMessage(
-		  FORMAT_MESSAGE_ALLOCATE_BUFFER |
-		  FORMAT_MESSAGE_FROM_SYSTEM |
-		  FORMAT_MESSAGE_IGNORE_INSERTS,
-		  NULL, dw,
-		  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lp_msg_buf,
-		  0,
-		  NULL);
+			FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM |
+			FORMAT_MESSAGE_IGNORE_INSERTS,
+			NULL, dw,
+			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lp_msg_buf,
+			0,
+			NULL);
 
 		LPVOID lp_display_buf = (LPVOID) LocalAlloc(
-		  LMEM_ZEROINIT,
-		  (lstrlen((LPCTSTR) lp_msg_buf) + lstrlen((LPCTSTR) __FILE__) + 40)
-		  * sizeof(TCHAR));
+			LMEM_ZEROINIT,
+			(lstrlen((LPCTSTR) lp_msg_buf) + lstrlen((LPCTSTR) __FILE__) + 40)
+			* sizeof(TCHAR));
 		_snprintf((LPTSTR) lp_display_buf,
 		          LocalSize(lp_display_buf) / sizeof(TCHAR),
 		          TEXT("%s failed with error %lu: %s"), __FILE__, dw,
